"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.compile = void 0;

var get = function get(obj, path) {
  for (var i = 0, path = path.split("."), len = path.length; i < len; i++) {
    obj = obj[path[i]];
  }

  return obj;
};

var _firstChild = function _firstChild(node) {
  return node.children[0];
};

var _identity = function _identity(node) {
  return node.value;
};

function _extractComparison(node) {
  var symbol = null;
  var value = null;
  node.children.forEach(function (child) {
    if (child.type === "SYMBOL") {
      if (symbol) {
        throw new Error("JS: You can only specify one symbol in a comparison.");
      }

      symbol = child;
    } else {
      if (value) {
        throw new Error("JS: You can only specify one value in a comparison.");
      }

      value = child;
    }
  });

  if (!(symbol && value)) {
    throw new Error("JS: Invalid comparison, could not find both symbol and value.");
  }

  return {
    symbol: symbol,
    value: value
  };
}

var generators = {
  NUMBER: _identity,
  BOOLEAN: _identity,
  PRIMITIVE: _identity,
  STRING: _identity,
  SYMBOL: function SYMBOL(node) {
    if (node.alone) {
      return function (obj) {
        return get(obj, node.value) !== undefined;
      };
    }

    return node.value;
  },
  "-": function _(node) {
    return -_firstChild(node).value;
  },
  "&&": function _(node) {
    var conditions = node.children.map(function (_node) {
      return _processNode(_node);
    });
    return function (obj) {
      return conditions.every(function (fn) {
        return fn(obj);
      });
    };
  },
  "||": function _(node) {
    var conditions = node.children.map(function (_node) {
      return _processNode(_node);
    });
    return function (obj) {
      return conditions.some(function (fn) {
        return fn(obj);
      });
    };
  },
  ARRAY: function ARRAY(node) {
    return node.children.map(_processNode);
  },
  IN: function IN(node) {
    var field = _processNode(node.children[0]);

    var valueList = _processNode(node.children[1]);

    return function (obj) {
      return valueList.includes(get(obj, field));
    };
  },
  "!": function _(node) {
    var expr = _processNode(node.children[0]);

    return function (obj) {
      return !expr(obj);
    };
  },
  "==": function _(node) {
    var comparison = _extractComparison(node);

    var symbol = _processNode(comparison.symbol);

    var value = _processNode(comparison.value);

    return function (obj) {
      return get(obj, symbol) === value;
    };
  },
  "!=": function _(node) {
    var comparison = _extractComparison(node);

    var symbol = _processNode(comparison.symbol);

    var value = _processNode(comparison.value);

    return function (obj) {
      return get(obj, symbol) !== value;
    };
  },
  MATCH: function MATCH(node) {
    var comparison = _extractComparison(node);

    var symbol = _processNode(comparison.symbol);

    var value = _processNode(comparison.value);

    var regex = new RegExp(value);
    return function (obj) {
      return get(obj, symbol).match(regex);
    };
  },
  "<": function _(node) {
    var comparison = _extractComparison(node);

    var symbol = _processNode(comparison.symbol);

    var value = _processNode(comparison.value);

    return function (obj) {
      return get(obj, symbol) < value;
    };
  },
  "<=": function _(node) {
    var comparison = _extractComparison(node);

    var symbol = _processNode(comparison.symbol);

    var value = _processNode(comparison.value);

    return function (obj) {
      return get(obj, symbol) <= value;
    };
  },
  ">": function _(node) {
    var comparison = _extractComparison(node);

    var symbol = _processNode(comparison.symbol);

    var value = _processNode(comparison.value);

    return function (obj) {
      return get(obj, symbol) > value;
    };
  },
  ">=": function _(node) {
    var comparison = _extractComparison(node);

    var symbol = _processNode(comparison.symbol);

    var value = _processNode(comparison.value);

    return function (obj) {
      return get(obj, symbol) >= value;
    };
  },
  EXPRESSION: function EXPRESSION(node) {
    var conditions = node.children.map(function (_node) {
      return _processNode(_node);
    });
    return function (obj) {
      return conditions.every(function (fn) {
        return fn(obj);
      });
    };
  }
};

var _processNode = function _processNode(node) {
  if (!(node.type in generators)) {
    throw new Error("Invalid node type ".concat(node.type));
  }

  return generators[node.type](node);
};

var compile = function compile(tree) {
  return _processNode(tree);
};

exports.compile = compile;
var _default = compile;
exports["default"] = _default;
//# sourceMappingURL=index.js.map